<?php

namespace Quivers\Integration\Model\Config\Source;
use Quivers\Integration\LogTracking;

class ClaimingGroupConfig implements \Magento\Framework\Option\ArrayInterface
{
	protected $curlClient;
	protected $scopeConfig;
	protected $baseUrl;

	protected $baseTestUrl = "https://cloudhub.quiversdemo.com/";
	protected $baseProdUrl = "https://cloudhub-internal.quivers.com/";
	protected $claim_url = "/api/v1/private/ClaimingPolicies/GetByBusiness?refId=";

    const XML_QUIVERS_API_KEY = 'quivers/parameters/quivers_api_key';
    const XML_QUIVERS_BUSINESS_REFID = 'quivers/parameters/merchant_ref_id';
    const XML_QUIVERS_SANDBOX = 'quivers/parameters/quivers_sanndbox';

	public function __construct(
		\Magento\Framework\HTTP\Client\Curl $curl,
		\Magento\Framework\App\Request\Http $request,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
	) {
		$this->curlClient = $curl;
		$this->request = $request;
		$this->scopeConfig = $scopeConfig;
	}

	public function getClaimingGroup() {
		$websiteScope = \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE;
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $websiteId = (int) $this->request->getParam('website',0);
        $storeId = (int) $this->request->getParam('store',0);
		$debug_mod = $this->scopeConfig->getValue('quivers/parameters/quivers_debug_mode', $storeScope);
        $log = new LogTracking($this->scopeConfig);
		if($debug_mod){
            $log->debug_mode_on(
				$debug_mod,'WebsiteId - '.$websiteId.' And StoreId - '.$storeId,
				null,null,null); 
        }
		if($websiteId){
			$quiversAPIKey = $this->scopeConfig->getValue(self::XML_QUIVERS_API_KEY, $websiteScope, $websiteId);
            $quiversSandbox = $this->scopeConfig->getValue(self::XML_QUIVERS_SANDBOX, $websiteScope, $websiteId);
        	$quiversBusinessRefId = $this->scopeConfig->getValue(self::XML_QUIVERS_BUSINESS_REFID, $websiteScope, $websiteId);
		}else{
			$quiversAPIKey = $this->scopeConfig->getValue(self::XML_QUIVERS_API_KEY, $storeScope, $storeId);
            $quiversSandbox = $this->scopeConfig->getValue(self::XML_QUIVERS_SANDBOX, $storeScope, $storeId);
        	$quiversBusinessRefId = $this->scopeConfig->getValue(self::XML_QUIVERS_BUSINESS_REFID, $storeScope, $storeId);
		}
		// get claimng group places by calling the claiming group place api
		if($quiversSandbox==1) $this->baseUrl = $this->baseTestUrl;
		else $this->baseUrl = $this->baseProdUrl;

		$profileUrl = $this->baseUrl.$this->claim_url.$quiversBusinessRefId;
        $this->curlClient->addHeader("Content-Type","application/json");
        $this->curlClient->addHeader("Authorization","apikey ".$quiversAPIKey);
        $this->curlClient->get($profileUrl);
		$response = json_decode($this->curlClient->getBody());
		$claimingGroups = [];
		if($this->curlClient->getStatus()==200 && is_array($response)) {
			if(count($response)>0) {
				foreach($response as $res) {
						if($res->Inclusive)
						{
						array_push($claimingGroups, (object)['id' => $res->Id, 'name' => $res->Name ]);
						}
				}
			}
		}
		return $claimingGroups;
	}

	public function toOptionArray() {
		$allclaimingGroups = $this->getClaimingGroup();
		$options = [];
		if(count($allclaimingGroups)>0) {
			foreach($allclaimingGroups as $cGroup) {
				$options[] = ['value' => $cGroup->id, 'label' => __($cGroup->name)];
			}
		}
		return $options;
	}
}