<?php
namespace Quivers\Integration\Model;

use Quivers\Integration\LogTracking;
// require __DIR__  . '/vendor/autoload.php';
// use PayPal;


class QuiversPayPal extends \Magento\Payment\Model\Method\Cc
{

    protected $_code = 'quivers_integration_paypal';
 
    protected $_canAuthorize = true;
    protected $_isGateway = true;

    protected $_scopeConfig;
    protected $sandbox;
    protected $clientId;
    protected $clientSecret;

    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory,
        \Magento\Framework\Api\AttributeValueFactory $customAttributeFactory,
        \Magento\Payment\Helper\Data $paymentData,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Payment\Model\Method\Logger $logger,
        \Magento\Framework\Module\ModuleListInterface $moduleList,
        \Magento\Framework\Stdlib\DateTime\TimezoneInterface $localeDate
    ) {
        parent::__construct(
            $context,
            $registry,
            $extensionFactory,
            $customAttributeFactory,
            $paymentData,
            $scopeConfig,
            $logger,
            $moduleList,
            $localeDate
        );
        $this->_scopeConfig = $scopeConfig;
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $this->clientId = $this->_scopeConfig->getValue("payment/quivers_integration_paypal/client_id", $storeScope);
        $this->clientSecret = $this->_scopeConfig->getValue("payment/quivers_integration_paypal/client_secret", $storeScope);
        $this->sandbox = $this->_scopeConfig->getValue("payment/quivers_integration_paypal/quivers_paypal_sanndbox", $storeScope); 
    }

    public function assignData(\Magento\Framework\DataObject $data)
    {
        parent::assignData($data);

        $additionalData = $data->getAdditionalData();
        if (is_array($additionalData))
            $data->setData(array_merge($data->getData(), $additionalData));

        $info = $this->getInfoInstance();
        $payerId = $data['cc_paypal_payer_id'];
        $paymentId = $data['cc_paypal_payment_id'];
        $info->setAdditionalInformation('cc_paypal_payer_id', $payerId);
        $info->setAdditionalInformation('cc_paypal_payment_id', $paymentId);
        return $this;
    }
 
    public function authorize(\Magento\Payment\Model\InfoInterface $payment, $amount)
    {
        $startTime = microtime(true);
        $log = new LogTracking($this->_scopeConfig);
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
    	$paymentId = $payment->getAdditionalInformation('cc_paypal_payment_id');
    	$payerId = $payment->getAdditionalInformation('cc_paypal_payer_id');
        $debug_mod =$this->_scopeConfig->getValue('quivers/parameters/quivers_debug_mode', $storeScope);

        //validate grand total
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance(); 
        $checkout_session=$objectManager->get('\Magento\Checkout\Model\Session');
        $quote = $checkout_session->getQuote();
        $email=$quote->getBillingAddress()->getEmail();
        $grandTotal=$payment->getAdditionalInformation('QuiversTotal');
        if(!$grandTotal){
            $grandTotal= $amount;
        }

         #store credit code
         $cartTotalRepository = $objectManager->get('\Magento\Quote\Api\CartTotalRepositoryInterface');
         $totals = $cartTotalRepository->get($quote->getId());
         $storeCredit = 0.0;
         $totalSegmentArray = [];
         foreach($totals->getTotalSegments() as $totalSegment){
             $totalSegmentArray = $totalSegment->toArray();
             if ($totalSegmentArray["code"] == "customerbalance"){
                 $storeCredit = $totalSegmentArray["value"];
             }
         }
         if (is_numeric($storeCredit)){
             if($storeCredit<0){
                 $grandTotal = $grandTotal + $storeCredit;
             }
             elseif($storeCredit>0){
                 $grandTotal = $grandTotal - $storeCredit;
             }
         }
         else{
             str_replace("-","",$storeCredit);
             $storeCredit = floatval($storeCredit);
             $grandTotal = $grandTotal - $storeCredit;
         }
         
        $order = $payment->getOrder();
        $billing = $order->getBillingAddress();
        $customerEmail = $billing->getEmail();
        $currency = $order->getOrderCurrencyCode();
        //$tax_amount = $order->getTaxAmount();
        // charge amount should be grand total(converted price)

        // $chargeAmount = $order->getGrandTotal();
        if($debug_mod){
            $log->debug_mode_on($debug_mod,
                'paypal  before apiContext check QuiversPaypal.php- '.$customerEmail,
                null, null ,["amount"=>$amount,"clientId"=>$this->clientId,"clientSecret"=>$this->clientSecret ]);
        }
        
        // $chargeAmount = $amount;

        $chargeAmount =$grandTotal;
            //debug mode
        $LogData = [
            "amount" => $amount,
            "quivers_grand_total" => $payment->getAdditionalInformation('QuiversTotal'),
            "final_charge_amount" => $chargeAmount,
            "storeCredit" => $storeCredit,
            "totalSegmentArray"=>$totalSegmentArray,
            "currency"=> $currency
        ];

        $log->debug_mode_on(false,
        'QuiversPayPal.php charge amount - '.$email,
        null, null ,$LogData);

        $apiContext = new \PayPal\Rest\ApiContext(
                new \PayPal\Auth\OAuthTokenCredential(
                    $this->clientId,     // ClientID
                    $this->clientSecret      // ClientSecret
                )
        );

        $mode = 'sandbox';
        if($this->sandbox != 1) {
            $mode = 'live';
        }

        $apiContext->setConfig(
              array(
                'mode' => $mode,
              )
        );
        
        try {
            $paypalPayment = \PayPal\Api\Payment::get($paymentId, $apiContext);
        } 
        catch(\PayPal\Exception\PayPalConnectionException $ex) {
            $log->debug_mode_on($debug_mod,
			'Catch  "QuiversPaypal.php -"Connection Failed: Unable to retrieve payment. '.$quote->getBillingAddress()->getEmail(),
			null,  null,$ex->getMessage());
        	throw new \Magento\Framework\Validator\Exception(__('Connection Failed: Unable to retrieve payment. '.$ex->getMessage()));
        }
        catch (\Exception $ex) {
            $log->debug_mode_on($debug_mod,
			'Catch  "QuiversPaypal.php -"Unable to retrieve payment. '.$quote->getBillingAddress()->getEmail(),
			null,  null,$ex->getMessage());
        	throw new \Magento\Framework\Validator\Exception(__('Unable to retrieve payment. '.$ex->getMessage()));
        }
        // ### Payment Execute
        // PaymentExecution object includes information necessary
        // to execute a PayPal account payment.
        $execution = new \PayPal\Api\PaymentExecution();
        $execution->setPayerId($payerId);
        // ### Optional Changes to Amount
        // If you wish to update the amount that you wish to charge the customer,
        // based on the shipping address or any other reason, you could
        // do that by passing the transaction object with just `amount` field in it.
        $transaction = new \PayPal\Api\Transaction();
        $paypalAmount = new \PayPal\Api\Amount();
        $paypalAmount->setCurrency($currency);
        $paypalAmount->setTotal($chargeAmount);
        $transaction->setAmount($paypalAmount);
        // Add the above transaction object inside our Execution object.
        $execution->addTransaction($transaction);
        $authorizationId = false;
        try {
            // Execute the payment
            $result = $paypalPayment->execute($execution, $apiContext);
            $authorizationId = $this->getAuthorizationId($result);
        }
        catch(\PayPal\Exception\PayPalConnectionException $ex) {
            $log->debug_mode_on($debug_mod,
			'Catch  "QuiversPaypal.php -"Connection Failed: Unable to execute payment.  '.$quote->getBillingAddress()->getEmail(),
			null,  null,$ex->getMessage());
        	throw new \Magento\Framework\Validator\Exception(__('Connection Failed: Unable to execute payment. '.$ex->getMessage()));
            } 
        catch (\Exception $ex) {
            $log->debug_mode_on($debug_mod,
			'Catch  "QuiversPaypal.php -"Unable to execute payment.  '.$quote->getBillingAddress()->getEmail(),
			null,  null,$ex->getMessage());
        	throw new \Magento\Framework\Validator\Exception(__('Unable to execute payment. '.$ex->getMessage()));
        }
        if(!$authorizationId) {
            $log->debug_mode_on($debug_mod,
			'Catch  "QuiversPaypal.php -"Unable to authorize payment.  '.$quote->getBillingAddress()->getEmail(),
			null,  null,$ex->getMessage());
            throw new \Magento\Framework\Validator\Exception(__('Unable to authorize payment.'));
        }
        $order_data = ["Amount"=>$chargeAmount,'email'=>$email];
        $log->payment_request($order_data, "PayPal", date('r', $startTime)."UTC", ["Transaction Id"=>$authorizationId], "success");
        $payment
            ->setTransactionId($authorizationId)
            ->setIsTransactionClosed(0);
        
        return $this;
    }
 
   public function isAvailable(\Magento\Quote\Api\Data\CartInterface $quote = null)
    {	

        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $secret_key = $this->_scopeConfig->getValue("payment/quivers_integration_paypal/client_secret", $storeScope);
        $activekey = $this->_scopeConfig->getValue("payment/quivers_integration_paypal/active", $storeScope);
        $active = $this->_scopeConfig->getValue("quivers/parameters/active", $storeScope);
        if (!$secret_key or $active==0) {
            return false;
        }
        return $activekey;
    }
    
    public function canUseForCurrency($currencyCode)
    {
        return true;
    }

    public function validate()
    {
        // validation is already handled by paypal
        return true;
    }

    protected function getAuthorizationId($paypalPayment) {
        $transactions = $paypalPayment->getTransactions();
        foreach ($transactions as $transaction) {
            $relatedResources = $transaction->getRelatedResources();
            foreach ($relatedResources as $relatedResource) {
                $authorization = $relatedResource->getAuthorization();
                if($authorization) {
                    $authorizationId = $authorization->getId();
                    return $authorizationId;
                }
            }
        }

        return false;
    }

}
